@@ -6,7 +6,7 @@ from six import iteritems
 __all__ = []
 
 _DEFAULTS = {
-    'CAS_ADMIN_PREFIX': None,
+    'CAS_ADMIN_AUTH': True,
     'CAS_EXTRA_LOGIN_PARAMS': None,
     'CAS_IGNORE_REFERER': False,
     'CAS_LOGOUT_COMPLETELY': True,
