@@ -7,8 +7,10 @@ from six.moves.urllib.request import urlopen
 
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
-from django_cas.models import User, Tgt, PgtIOU
+from django_cas.models import Tgt, PgtIOU
 from django_cas.utils import cas_response_callbacks
+from django.contrib.auth import get_user_model
+
 
 __all__ = ['CASBackend']
 
@@ -33,8 +35,16 @@ def _verify_cas1(ticket, service):
         page.close()
 
 
-def _verify_cas2(ticket, service):
-    """Verifies CAS 2.0+ XML-based authentication ticket.
+def _verify_cas2(ticket, service, sufix='proxyValidate'):
+    return _internal_verify_cas(ticket, service, sufix)
+
+
+def _verify_cas3(ticket, service, sufix='p3/proxyValidate'):
+    return _internal_verify_cas(ticket, service, sufix)
+
+
+def _internal_verify_cas(ticket, service, sufix):
+    """Verifies CAS 2.0 and 3.0  XML-based authentication ticket.
 
     Returns username on success and None on failure.
     """
@@ -49,7 +59,7 @@ def _verify_cas2(ticket, service):
     else:
         params = {'ticket': ticket, 'service': service}
 
-    url = (urljoin(settings.CAS_SERVER_URL, 'proxyValidate') + '?' +
+    url = (urljoin(settings.CAS_SERVER_URL, sufix) + '?' +
            urlencode(params))
 
     page = urlopen(url)
@@ -106,7 +116,7 @@ def verify_proxy_ticket(ticket, service):
     finally:
         page.close()
 
-_PROTOCOLS = {'1': _verify_cas1, '2': _verify_cas2}
+_PROTOCOLS = {'1': _verify_cas1, '2': _verify_cas2, '3': _verify_cas3}
 
 if settings.CAS_VERSION not in _PROTOCOLS:
     raise ValueError('Unsupported CAS_VERSION %r' % settings.CAS_VERSION)
@@ -120,22 +130,23 @@ class CASBackend(object):
     supports_object_permissions = False
     supports_inactive_user = False
 
-    def authenticate(self, ticket, service):
-        """Verifies CAS ticket and gets or creates User object
+    def authenticate(self, request, ticket, service):
+        """Verifies CAS ticket and gets or create User object
             NB: Use of PT to identify proxy
         """
 
         username = _verify(ticket, service)
+        user_model = get_user_model()
         if not username:
             return None
         try:
             if settings.CAS_USERNAME_FORMAT:
                 username = settings.CAS_USERNAME_FORMAT(username)
-            user = User.objects.get(username=username)
-        except User.DoesNotExist:
+            user = user_model.objects.get(username=username)
+        except user_model.DoesNotExist:
             if settings.CAS_USER_CREATION:
                 # user will have an "unusable" password
-                user = User.objects.create_user(username, '')
+                user = user_model.objects.create_user(username, '')
                 user.save()
             else:
                 return None
@@ -144,7 +155,8 @@ class CASBackend(object):
     def get_user(self, user_id):
         """Retrieve the user's entry in the User model if it exists"""
 
+        user_model = get_user_model()
         try:
-            return User.objects.get(pk=user_id)
-        except User.DoesNotExist:
+            return user_model.objects.get(pk=user_id)
+        except user_model.DoesNotExist:
             return None
